/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): Romain Lenglet
 */

package org.objectweb.dream.aggregator;

import org.objectweb.dream.Pull;
import org.objectweb.dream.PullException;
import org.objectweb.dream.dreamannotation.DreamComponent;
import org.objectweb.dream.message.Message;
import org.objectweb.dream.message.MessageManagerType;
import org.objectweb.fractal.fraclet.annotation.annotations.Attribute;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;

/**
 * Message aggregator, when a message is asked in the <code>out-pull</code>
 * server interface, this component pulls messages on its <code>in-pull</code>
 * client interface until <code>null</code> is returned, and then returns an
 * aggregated message containing every pulled messages.
 */
@DreamComponent(controllerDesc = "dreamPrimitive")
@Provides(interfaces=@Interface(name = "out-pull", signature=Pull.class))
public class PullPullAggregatorWhileNotNull
    implements
      Pull
{
  // ---------------------------------------------------------------------------
  // Attributes fields
  // ---------------------------------------------------------------------------

  /**
   * Defines the behavior when the first pulled message is null.
   */
  @Attribute(argument = "pullEmptyMessagePolicy")
  boolean                  pullEmptyMessagePolicy;

  // ---------------------------------------------------------------------------
  // Client interfaces
  // ---------------------------------------------------------------------------

  @Requires(name = "in-pull")
  protected Pull           inPullItf;
  @Requires(name = "message-manager")
  protected MessageManagerType messageManagerItf;

  // ---------------------------------------------------------------------------
  // Implementation of the Pull interface
  // ---------------------------------------------------------------------------

  /**
   * @see Pull#pull()
   */
  public Message pull() throws PullException
  {
    Message inMessage = inPullItf.pull();
    if (inMessage == null && (!pullEmptyMessagePolicy))
    {
      return null;
    }
    Message outMessage = messageManagerItf.createMessage();
    while (inMessage != null)
    {
      messageManagerItf.addSubMessage(outMessage, inMessage);
      inMessage = inPullItf.pull();
    }
    return outMessage;
  }
}