/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.protocol;

import org.objectweb.dream.PushException;
import org.objectweb.dream.message.Message;

/**
 * Push interface on which <i>incoming messages</i> are passed to a protocol.
 * Incoming messages are messages going up the protocol stack.
 * 
 * @see OutgoingPush
 * @see org.objectweb.dream.Push
 */
public interface IncomingPush
{
  
  /**
   * The commonly used name to refer to this interface when it is used as an
   * output.
   */
  String OUT_PUSH_ITF_NAME          = "incoming-out-push";

  /**
   * The commonly used name to refer to this interface when it is used as an
   * input.
   */
  String IN_PUSH_ITF_NAME           = "incoming-in-push";

  /**
   * Gives an incoming message to a protocol.
   * 
   * @param message the message to give to the component.
   * @throws PushException if an exception occurs.
   */
  void incomingPush(Message message) throws PushException;

  /**
   * Notifies the session, this interface belong, that the lower session
   * accessible through the given interface has been unexpectedly closed.
   * 
   * @param outgoingPush the interface provided by the closed session.
   * @param exception the exception that forced the session to close. May be
   *          <code>null</code>.
   */
  void incomingClosed(Object outgoingPush, Exception exception);
}
