/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): Pierre Garcia 
 */

package org.objectweb.dream.protocol;

import java.util.Map;

/**
 * Base protocol interface, contains commonly used names to refer to protocol
 * interfaces.
 * Note : Annotation @Interface is not used because this interface has not a
 * unique name. It depends on the related role of the interface
 */
public interface Protocol
{
  /** The default name of this interface */
  String ITF_NAME                = "protocol";

  /**
   * The default name of a client interface of a protocol component bound to a
   * lower level protocol.
   */
  String LOWER_PROTOCOL_ITF_NAME = "lower-protocol";

  /**
   * Creates a new export identifier with the provided info
   * 
   * @param info the info used to create the export identifier.
   * @param next the export identifier of the lower level protocols.
   * @return a new export identifier.
   * @throws InvalidExportIdentifierException if the export identifier can't be
   *           created.
   */
  ExportIdentifier createExportIdentifier(Map<String, Object> info,
      ExportIdentifier[] next) throws InvalidExportIdentifierException;
}
