/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact : dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): Vivien Quema, Romain Lenglet
 */

package org.objectweb.dream.protocol.atomicity;

import org.objectweb.dream.Push;
import org.objectweb.dream.PushException;
import org.objectweb.dream.dreamannotation.DreamComponent;
import org.objectweb.dream.message.Message;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;

/**
 * A Router that choose a route depending on the pushing thread. This component
 * provides an interface to set the thread that is considered as the reacting
 * thread.
 */
@DreamComponent(controllerDesc = "dreamPrimitive")
@Provides(interfaces = { @Interface(name = "in-push", signature = Push.class) })
public class AtomicRouter implements Push, SetReactorThread {

    // ------------------------------------------------------------------------
    // ---
    // Client Interfaces
    // ------------------------------------------------------------------------
    // ---

    /**
     * the name of the client interface uesd to push messages if the pushing
     * thread is the reacting thred
     */
    public static final String OUT_REACT_PUSH_ITF_NAME = "outReactPush";

    @Requires(name = OUT_REACT_PUSH_ITF_NAME)
    private Push outReactPushItf;

    /**
     * The name of the client interface used to push messages if the pushing
     * thread is not the reacting thread
     */
    public static final String OUT_NOT_REACT_PUSH_ITF_NAME = "outNotReactPush";

    @Requires(name = OUT_NOT_REACT_PUSH_ITF_NAME)
    private Push outNotReactPushItf;

    // ------------------------------------------------------------------------
    // ---
    // Fields
    // ------------------------------------------------------------------------
    // ---

    private Thread reactorThread;

    // ------------------------------------------------------------------------
    // ---
    // Push Interface Implementation
    // ------------------------------------------------------------------------
    // ---

    /**
     * @see Push#push(Message)
     */
    public void push(Message message) throws PushException {
        if (Thread.currentThread() == reactorThread) {
            outReactPushItf.push(message);
        } else {
            outNotReactPushItf.push(message);
        }
    }

    // ------------------------------------------------------------------------
    // ---
    // SetReactorThread Interface Implementation
    // ------------------------------------------------------------------------
    // ---

    /**
     * @see SetReactorThread#setReactorThread(Thread)
     */
    public void setReactorThread(Thread reactorThread) {
        this.reactorThread = reactorThread;
    }
}
