/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact : dream@objectweb.org
 *
 * Initial developer(s): Vivien Quema
 * Contributor(s): Romain Lenglet
 */

package org.objectweb.dream.pushwithreturn;


import org.objectweb.dream.Push;
import org.objectweb.dream.PushException;
import org.objectweb.dream.message.MessageManagerType;
import org.objectweb.dream.dreamannotation.DreamComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;

/**
 * Basic implementation of the {@link HandleReturnMessageItf}interface. The
 * component simply forwards return messages.
 */
@DreamComponent(controllerDesc = "dreamPrimitive")
public class HandleReturnMessage 
    implements
      HandleReturnMessageItf
{
  // ---------------------------------------------------------------------------
  // Client interfaces
  // ---------------------------------------------------------------------------

  /** The client interface used to access the message manager. */
  @Requires(name = "message-manager")
  MessageManagerType messageManagerItf;

  /** The Push output interface. */
  @Requires(name = "out-push")
  Push               outPushItf;

  // ---------------------------------------------------------------------------
  // Implementation of the HandleReturnMessageItf interface.
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.dream.pushwithreturn.HandleReturnMessageItf#handleReturnMessage(org.objectweb.dream.pushwithreturn.WaitingKey)
   */
  public void handleReturnMessage(WaitingKey waitingKey) throws Exception
  {
    try
    {
      outPushItf.push(waitingKey.getReturnMessage());
      messageManagerItf.deleteMessage(waitingKey.getMessage());
    }
    catch (PushException e)
    {
      throw new Exception(e);
    }
  }
}