/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): Romain Lenglet
 */

package org.objectweb.dream.queue;

import org.objectweb.dream.message.Message;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;

/**
 * Calculates keys for messages. Keys are {@link Comparable} objects that can be
 * used to order messages in a buffer.
 * 
 * @see MessageKeyManager
 */
@Interface(name = MessageComparableKeyManager.ITF_NAME)
public interface MessageComparableKeyManager
{

  /**
   * The commonly used name to refer to an interface with this signature.
   */
  String ITF_NAME = "message-comparable-key-manager";

  /**
   * Returns a new empty comparable key object.
   * 
   * @return a new empty comparable key object; never <code>null</code>.
   */
  Comparable createComparableKey();

  /**
   * Fills the given comparable key to match the given message.
   * 
   * @param key the comparable key to calculate; must not be <code>null</code>.
   * @param message the message to which the comparable key will be associated;
   *          must not be <code>null</code>
   */
  void fillComparableKey(Comparable key, Message message);

  /**
   * Clones the given comparable key.
   * 
   * @param key the comparable key to clone; never <code>null</code>.
   * @return a clone of the given comparable key; never <code>null</code>.
   */
  Comparable duplicateComparableKey(Comparable key);

  /**
   * Delete the given comparable key instance.
   * 
   * @param key the comparable key to delete; never <code>null</code>.
   */
  void deleteComparableKey(Comparable key);

}