/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Pierre Garcia
 * Contributor(s):
 */

package org.objectweb.dream.queue.keyed.handler.outgoing;


import org.objectweb.dream.PullException;
import org.objectweb.dream.message.Message;
import org.objectweb.dream.queue.keyed.KeyedPull;
import org.objectweb.dream.queue.keyed.buffer.KeyedBufferRemove;
import org.objectweb.dream.dreamannotation.DreamComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;

/**
 * An abstract keyed incoming handler that implements the {@link org.objectweb.dream.queue.keyed.KeyedPull}
 * interface. "Keyed incoming handler" developers can inherit this class. They
 * must implement the
 * {@link org.objectweb.dream.queue.keyed.KeyedPull#pull(java.lang.Object) pull} method
 * 
 * @author pgarcia
 */
@DreamComponent(controllerDesc = "dreamUnstoppablePrimitive")
@Provides(interfaces = @Interface(name = KeyedPull.OUT_KEYED_PULL_ITF_NAME, signature = KeyedPull.class))
public abstract class AbstractKeyedOutgoingHandler 
  
  implements KeyedPull
{
  // ---------------------------------------------------------------------------
  // Client interfaces.
  // ---------------------------------------------------------------------------

  /**
   * The buffer into which pushed messages are added.
   */
  @Requires(name = KeyedBufferRemove.ITF_NAME)
  protected KeyedBufferRemove keyedBufferRemoveItf;

  // ---------------------------------------------------------------------------
  // Abstract method
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.dream.queue.keyed.KeyedPull#pull(java.lang.Object)
   */
  public abstract Message pull(Object key) throws PullException;

}
