/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact : dream@objectweb.org
 *
 * Initial developer(s): Vivien Quema
 * Contributor(s):
 */

package org.objectweb.dream.serializator;

import java.io.IOException;

import org.objectweb.dream.message.Message;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;

/**
 * A deserializator defines a method <code>deserialize</code> that takes an
 * object of type T (example : byte array) as parameter and returns the
 * corresponding message.
 */
@Interface(name = DeSerializatorItf.ITF_NAME)
public interface DeSerializatorItf<T>
{

  /** The default name of the {@link DeSerializatorItf }interface. */
  String ITF_NAME = "deserializator";

  /**
   * Deserializes the given object and returns a message.
   * 
   * @param objectToDecode the object to be deserialized.
   * @return a message.
   * @throws IOException if an exception occurs while serializing the message.
   */
  Message deserialize(T objectToDecode) throws IOException;
}